import os

script_path = os.path.dirname(os.path.realpath(__file__))

IMAGES = ['%s/resources/1.jpg' % script_path,
          '%s/resources/2.jpg' % script_path,
          '%s/resources/3.jpg' % script_path,
          '%s/resources/4.jpg' % script_path,
          '%s/resources/5.jpg' % script_path,
          '%s/resources/6.jpg' % script_path,
          '%s/resources/7.jpg' % script_path]

VIDEOS = ['%s/resources/1.flv' % script_path, ]

PDFS = ['%s/resources/blank_pdf_document.pdf' % script_path, ]

CSVS = ['%s/resources/blank_csv.csv' % script_path, ]

TXTS = ['%s/resources/blank_text_document.txt' % script_path, ]
